@@ -2,17 +2,7 @@
 #
 # Copyright 2016 Hewlett-Packard Enterprise
 #
-# Licensed under the Apache License, Version 2.0 (the "License"); you may
-# not use this file except in compliance with the License. You may obtain
-# a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
-# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
-# License for the specific language governing permissions and limitations
-# under the License.
+# SPDX-License-Identifier: Apache-2.0
 
 import importlib
 import logging
@@ -85,7 +75,8 @@ class BanditConfigGeneratorTests(testtools.TestCase):
                 config[plugin.name] = module.gen_config(
                     function._takes_config)
         settings = config_generator.get_config_settings()
-        self.assertEqual(yaml.safe_dump(config), settings)
+        self.assertEqual(yaml.safe_dump(config, default_flow_style=False),
+                         settings)
 
     @mock.patch('sys.argv', ['bandit-config-generator', '--show-defaults'])
     def test_main_show_defaults(self):
