@@ -2,20 +2,9 @@
 #
 # Copyright 2015 Hewlett-Packard Development Company, L.P.
 #
-# Licensed under the Apache License, Version 2.0 (the "License"); you may
-# not use this file except in compliance with the License. You may obtain
-# a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
-# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
-# License for the specific language governing permissions and limitations
-# under the License.
+# SPDX-License-Identifier: Apache-2.0
 
 import os
-import sys
 
 import fixtures
 import mock
@@ -51,9 +40,9 @@ class ManagerTests(testtools.TestCase):
 
     def test_create_manager(self):
         # make sure we can create a manager
-        self.assertEqual(self.manager.debug, False)
-        self.assertEqual(self.manager.verbose, False)
-        self.assertEqual(self.manager.agg_type, 'file')
+        self.assertEqual(False, self.manager.debug)
+        self.assertEqual(False, self.manager.verbose)
+        self.assertEqual('file', self.manager.agg_type)
 
     def test_create_manager_with_profile(self):
         # make sure we can create a manager
@@ -61,9 +50,9 @@ class ManagerTests(testtools.TestCase):
                                   debug=False, verbose=False,
                                   profile=self.profile)
 
-        self.assertEqual(m.debug, False)
-        self.assertEqual(m.verbose, False)
-        self.assertEqual(m.agg_type, 'file')
+        self.assertEqual(False, m.debug)
+        self.assertEqual(False, m.verbose)
+        self.assertEqual('file', m.agg_type)
 
     def test_matches_globlist(self):
         self.assertTrue(manager._matches_glob_list('test', ['*tes*']))
@@ -71,24 +60,34 @@ class ManagerTests(testtools.TestCase):
 
     def test_is_file_included(self):
         a = manager._is_file_included(path='a.py', included_globs=['*.py'],
-                                      excluded_path_strings='',
+                                      excluded_path_strings=[],
                                       enforce_glob=True)
 
         b = manager._is_file_included(path='a.dd', included_globs=['*.py'],
-                                      excluded_path_strings='',
+                                      excluded_path_strings=[],
                                       enforce_glob=False)
 
         c = manager._is_file_included(path='a.py', included_globs=['*.py'],
-                                      excluded_path_strings='a.py',
+                                      excluded_path_strings=['a.py'],
                                       enforce_glob=True)
 
         d = manager._is_file_included(path='a.dd', included_globs=['*.py'],
-                                      excluded_path_strings='',
+                                      excluded_path_strings=[],
+                                      enforce_glob=True)
+
+        e = manager._is_file_included(path='x_a.py', included_globs=['*.py'],
+                                      excluded_path_strings=['x_*.py'],
+                                      enforce_glob=True)
+
+        f = manager._is_file_included(path='x.py', included_globs=['*.py'],
+                                      excluded_path_strings=['x_*.py'],
                                       enforce_glob=True)
         self.assertTrue(a)
         self.assertTrue(b)
         self.assertFalse(c)
         self.assertFalse(d)
+        self.assertFalse(e)
+        self.assertTrue(f)
 
     @mock.patch('os.walk')
     def test_get_files_from_dir(self, os_walk):
@@ -101,8 +100,8 @@ class ManagerTests(testtools.TestCase):
                                                included_globs=['*.py'],
                                                excluded_path_strings=None)
 
-        self.assertEqual(exc, set(['/a/c.ww']))
-        self.assertEqual(inc, set(['/a/a.py', '/a/b.py']))
+        self.assertEqual(set(['/a/c.ww']), exc)
+        self.assertEqual(set(['/a/a.py', '/a/b.py']), inc)
 
     def test_populate_baseline_success(self):
         # Test populate_baseline with valid JSON
@@ -143,10 +142,11 @@ class ManagerTests(testtools.TestCase):
     def test_results_count(self):
         levels = [constants.LOW, constants.MEDIUM, constants.HIGH]
         self.manager.results = (
-            [issue.Issue(severity=l, confidence=l) for l in levels])
+            [issue.Issue(severity=level, confidence=level)
+             for level in levels])
 
-        r = [self.manager.results_count(sev_filter=l, conf_filter=l)
-             for l in levels]
+        r = [self.manager.results_count(sev_filter=level, conf_filter=level)
+             for level in levels]
 
         self.assertEqual([3, 2, 1], r)
 
@@ -158,13 +158,10 @@ class ManagerTests(testtools.TestCase):
         conf_level = constants.LOW
         output_filename = os.path.join(temp_directory, "_temp_output")
         output_format = "invalid"
-        tmp_file = open(output_filename, 'w')
-        self.manager.output_results(lines, sev_level, conf_level, tmp_file,
-                                    output_format)
-        if sys.stdout.isatty():
-            self.assertFalse(os.path.isfile(output_filename))
-        else:
-            self.assertTrue(os.path.isfile(output_filename))
+        with open(output_filename, 'w') as tmp_file:
+            self.manager.output_results(lines, sev_level, conf_level,
+                                        tmp_file, output_format)
+        self.assertTrue(os.path.isfile(output_filename))
 
     def test_output_results_valid_format(self):
         # Test that output_results succeeds given a valid format
@@ -174,17 +171,17 @@ class ManagerTests(testtools.TestCase):
         conf_level = constants.LOW
         output_filename = os.path.join(temp_directory, "_temp_output.txt")
         output_format = "txt"
-        tmp_file = open(output_filename, 'w')
-        self.manager.output_results(lines, sev_level, conf_level, tmp_file,
-                                    output_format)
+        with open(output_filename, 'w') as tmp_file:
+            self.manager.output_results(lines, sev_level, conf_level,
+                                        tmp_file, output_format)
         self.assertTrue(os.path.isfile(output_filename))
 
     @mock.patch('os.path.isdir')
     def test_discover_files_recurse_skip(self, isdir):
         isdir.return_value = True
         self.manager.discover_files(['thing'], False)
-        self.assertEqual(self.manager.files_list, [])
-        self.assertEqual(self.manager.excluded_files, [])
+        self.assertEqual([], self.manager.files_list)
+        self.assertEqual([], self.manager.excluded_files)
 
     @mock.patch('os.path.isdir')
     def test_discover_files_recurse_files(self, isdir):
@@ -192,8 +189,8 @@ class ManagerTests(testtools.TestCase):
         with mock.patch.object(manager, '_get_files_from_dir') as m:
             m.return_value = (set(['files']), set(['excluded']))
             self.manager.discover_files(['thing'], True)
-            self.assertEqual(self.manager.files_list, ['files'])
-            self.assertEqual(self.manager.excluded_files, ['excluded'])
+            self.assertEqual(['files'], self.manager.files_list)
+            self.assertEqual(['excluded'], self.manager.excluded_files)
 
     @mock.patch('os.path.isdir')
     def test_discover_files_exclude(self, isdir):
@@ -201,8 +198,35 @@ class ManagerTests(testtools.TestCase):
         with mock.patch.object(manager, '_is_file_included') as m:
             m.return_value = False
             self.manager.discover_files(['thing'], True)
-            self.assertEqual(self.manager.files_list, [])
-            self.assertEqual(self.manager.excluded_files, ['thing'])
+            self.assertEqual([], self.manager.files_list)
+            self.assertEqual(['thing'], self.manager.excluded_files)
+
+    @mock.patch('os.path.isdir')
+    def test_discover_files_exclude_dir(self, isdir):
+        isdir.return_value = False
+
+        # Test exclude dir using wildcard
+        self.manager.discover_files(['./x/y.py'], True, './x/*')
+        self.assertEqual([], self.manager.files_list)
+        self.assertEqual(['./x/y.py'], self.manager.excluded_files)
+
+        # Test exclude dir without wildcard
+        isdir.side_effect = [True, False]
+        self.manager.discover_files(['./x/y.py'], True, './x/')
+        self.assertEqual([], self.manager.files_list)
+        self.assertEqual(['./x/y.py'], self.manager.excluded_files)
+
+        # Test exclude dir without wildcard or trailing slash
+        isdir.side_effect = [True, False]
+        self.manager.discover_files(['./x/y.py'], True, './x')
+        self.assertEqual([], self.manager.files_list)
+        self.assertEqual(['./x/y.py'], self.manager.excluded_files)
+
+        # Test exclude dir without prefix or suffix
+        isdir.side_effect = [False, False]
+        self.manager.discover_files(['./x/y/z.py'], True, 'y')
+        self.assertEqual([], self.manager.files_list)
+        self.assertEqual(['./x/y/z.py'], self.manager.excluded_files)
 
     @mock.patch('os.path.isdir')
     def test_discover_files_exclude_cmdline(self, isdir):
@@ -213,14 +237,22 @@ class ManagerTests(testtools.TestCase):
             m.assert_called_with('c', ['*.py', '*.pyw'], ['a', 'b'],
                                  enforce_glob=False)
 
+    @mock.patch('os.path.isdir')
+    def test_discover_files_exclude_glob(self, isdir):
+        isdir.return_value = False
+        self.manager.discover_files(['a.py', 'test_a.py', 'test.py'], True,
+                                    excluded_paths='test_*.py')
+        self.assertEqual(['a.py', 'test.py'], self.manager.files_list)
+        self.assertEqual(['test_a.py'], self.manager.excluded_files)
+
     @mock.patch('os.path.isdir')
     def test_discover_files_include(self, isdir):
         isdir.return_value = False
         with mock.patch.object(manager, '_is_file_included') as m:
             m.return_value = True
             self.manager.discover_files(['thing'], True)
-            self.assertEqual(self.manager.files_list, ['thing'])
-            self.assertEqual(self.manager.excluded_files, [])
+            self.assertEqual(['thing'], self.manager.files_list)
+            self.assertEqual([], self.manager.excluded_files)
 
     def test_run_tests_keyboardinterrupt(self):
         # Test that bandit manager exits when there is a keyboard interrupt
