@@ -1,16 +1,6 @@
 # Copyright (c) 2015 VMware, Inc.
 #
-#  Licensed under the Apache License, Version 2.0 (the "License"); you may
-#  not use this file except in compliance with the License. You may obtain
-#  a copy of the License at
-#
-#       http://www.apache.org/licenses/LICENSE-2.0
-#
-#  Unless required by applicable law or agreed to in writing, software
-#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
-#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
-#  License for the specific language governing permissions and limitations
-#  under the License.
+# SPDX-License-Identifier: Apache-2.0
 
 import collections
 import json
@@ -75,9 +65,9 @@ class JsonFormatterTests(testtools.TestCase):
         get_issue_list.return_value = collections.OrderedDict(
             [(self.issue, self.candidates)])
 
-        tmp_file = open(self.tmp_fname, 'w')
-        b_json.report(self.manager, tmp_file, self.issue.severity,
-                      self.issue.confidence)
+        with open(self.tmp_fname, 'w') as tmp_file:
+            b_json.report(self.manager, tmp_file, self.issue.severity,
+                          self.issue.confidence)
 
         with open(self.tmp_fname) as f:
             data = json.loads(f.read())
@@ -94,3 +84,5 @@ class JsonFormatterTests(testtools.TestCase):
                              data['results'][0]['line_range'])
             self.assertEqual(self.check_name, data['results'][0]['test_name'])
             self.assertIn('candidates', data['results'][0])
+            self.assertIn('more_info', data['results'][0])
+            self.assertIsNotNone(data['results'][0]['more_info'])
