@@ -1,17 +1,7 @@
 # Copyright (c) 2015 VMware, Inc.
 # Copyright (c) 2015 Hewlett Packard Enterprise
 #
-#  Licensed under the Apache License, Version 2.0 (the "License"); you may
-#  not use this file except in compliance with the License. You may obtain
-#  a copy of the License at
-#
-#       http://www.apache.org/licenses/LICENSE-2.0
-#
-#  Unless required by applicable law or agreed to in writing, software
-#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
-#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
-#  License for the specific language governing permissions and limitations
-#  under the License.
+# SPDX-License-Identifier: Apache-2.0
 
 import collections
 import tempfile
@@ -21,6 +11,7 @@ import testtools
 
 import bandit
 from bandit.core import config
+from bandit.core import docs_utils
 from bandit.core import issue
 from bandit.core import manager
 from bandit.formatters import text as b_text
@@ -45,7 +36,9 @@ class TextFormatterTests(testtools.TestCase):
                           format(_indent_val, _issue.severity.capitalize(),
                                  _issue.confidence.capitalize()),
                           "{}   Location: {}:{}".
-                          format(_indent_val, _issue.fname, _issue.lineno)]
+                          format(_indent_val, _issue.fname, _issue.lineno),
+                          "{}   More Info: {}".format(
+                              _indent_val, docs_utils.get_url(_issue.test_id))]
             if _code:
                 return_val.append("{}{}".format(_indent_val, _code))
             return '\n'.join(return_val)
@@ -74,8 +67,9 @@ class TextFormatterTests(testtools.TestCase):
         self.manager.out_file = self.tmp_fname
 
         get_issue_list.return_value = collections.OrderedDict()
-        tmp_file = open(self.tmp_fname, 'w')
-        b_text.report(self.manager, tmp_file, bandit.LOW, bandit.LOW, lines=5)
+        with open(self.tmp_fname, 'w') as tmp_file:
+            b_text.report(self.manager, tmp_file, bandit.LOW, bandit.LOW,
+                          lines=5)
 
         with open(self.tmp_fname) as f:
             data = f.read()
@@ -114,9 +108,9 @@ class TextFormatterTests(testtools.TestCase):
         with mock.patch(output_str_fn) as output_str:
             output_str.return_value = 'ISSUE_OUTPUT_TEXT'
 
-            tmp_file = open(self.tmp_fname, 'w')
-            b_text.report(self.manager, tmp_file, bandit.LOW, bandit.LOW,
-                          lines=5)
+            with open(self.tmp_fname, 'w') as tmp_file:
+                b_text.report(self.manager, tmp_file, bandit.LOW, bandit.LOW,
+                              lines=5)
 
             calls = [mock.call(issue_a, '', lines=5),
                      mock.call(issue_b, '', lines=5)]
@@ -125,9 +119,9 @@ class TextFormatterTests(testtools.TestCase):
 
         # Validate that we're outputting all of the expected fields and the
         # correct values
-        tmp_file = open(self.tmp_fname, 'w')
-        b_text.report(self.manager, tmp_file, bandit.LOW, bandit.LOW,
-                      lines=5)
+        with open(self.tmp_fname, 'w') as tmp_file:
+            b_text.report(self.manager, tmp_file, bandit.LOW, bandit.LOW,
+                          lines=5)
         with open(self.tmp_fname) as f:
             data = f.read()
 
@@ -179,9 +173,9 @@ class TextFormatterTests(testtools.TestCase):
         with mock.patch(output_str_fn) as output_str:
             output_str.return_value = 'ISSUE_OUTPUT_TEXT'
 
-            tmp_file = open(self.tmp_fname, 'w')
-            b_text.report(self.manager, tmp_file, bandit.LOW, bandit.LOW,
-                          lines=5)
+            with open(self.tmp_fname, 'w') as tmp_file:
+                b_text.report(self.manager, tmp_file, bandit.LOW, bandit.LOW,
+                              lines=5)
 
             calls = [mock.call(issue_a, '', lines=5),
                      mock.call(issue_b, '', show_code=False,
